# livelike_flutter_sdk

### The Official Flutter LiveLike SDK

[Documentation](https://docs.livelike.com/docs/getting-started-1)

[HomePage](https://livelike.com/)

#### Create Engagement SDK instance
```dart
final sdk = EngagementSDK("<client-id>");
```
#### Create Engagement SDK instance with access token support
```dart
final sdk = EngagementSDK.accessToken("<client-id>","<saved-access-token>",(accessToken){
      //fetched access token
});
```

#### Error Delegate Support
```dart
sdk.errorStream.listen((error) {
      print("Error: $error");
      //Getting error from any where from the sdk
    });
```


#### ChatView
```dart
ChatView(key: Key("${chatSession.chatRoomId}"),
         session:chatSession!);
```

#### WidgetView
```dart
WidgetView(session: session!,
           key: Key(session!.programId!));
```

#### Leaderboard API's

##### Getting Leaderboard associated with a Program
```dart
final List<LeaderBoard> list = await sdk.getLeaderBoards("<program-id>");
```

##### Getting leaderboard details
```dart
final LeaderBoard detail=await sdk.getLeaderBoardDetails("<leaderBoardId>");
```

##### Getting leaderboard entries
```dart
final List<LeaderBoardEntry> result = await sdk.getEntriesForLeaderBoard(
        "<leaderboard-id>", "<LiveLikePagination>");
```

##### Getting leaderboard entry for a given profile
```dart
final LeaderBoardEntry result = await sdk.getLeaderBoardEntryForProfile(
        "<leaderboard-id>", "<profile-id>");
```

##### Getting a leaderboard entry for the current user profile
```dart
final LeaderBoardEntry result = await sdk.getLeaderBoardEntryForCurrentUserProfile("<leaderboard-id>");
```

#### Sponsor APi
```dart
final sposnors =await sdk.sponsor.fetchByProgramId("<program-id>")
```

#### widgetListener Api
```dart
WidgetView(
        session: session,
        widgetListener: (widgetId, widgetKind) {
            print("object>> $widgetId => $widgetKind");
        },
)
```

#### Display Widget in WidgetView using LiveLikeWidget,Enable/Disable Widget Transition, Update State of widget inside WidgetView
```dart
final widgetKey = GlobalKey<WidgetViewState>();
    WidgetView(
        key:widgetKey
    )
//To display call the method
    widgetKey.currentState?.displayWidget(widget);
    widgetKey.currentState?.enableDefaultWidgetTransition(false);
    widgetKey.currentState?.setWidgetState(WidgetState.Result);
```
#### widgets have 4 states to ready,interacting,results,finished

#### Fetch User Interaction Data based on widgetId and WidgetKind
```dart
final interactionData = await session.fetchUserInteractionDataForWidget(widgetId!, widgetKind!);
```

## Note
#### If you are facing extra padding when keyboard appears in IOS, then add 
```dart
resizeToAvoidBottomInset: !Platform.IOS,
```
#### attribute to your Scaffold to avoid this issue, and make sure it is true for android platform

## Widget View with support in ListView
#### Now widgetView can be used with listview, you need to create an instance of widgetview and assign a key to it and then call the methods to perform the actions , below is the example of how to do this.
```dart

        final key = GlobalKey<WidgetViewState>();
        final widgetView = WidgetView(
          key: key,
          widgetListener: (id, kind) {
            // print("WidgetView>> $id >> $index");
          },
          showWidgetViewWithDefaultHeight: true,
          onWidgetViewInit: () {
            if (key.currentState != null) {
              await key.currentState!.enableDefaultWidgetTransition(false);
              await key.currentState!.showDismissButton(false);
              await key.currentState!.displayWidget(widget);
              await Future.delayed(const Duration(seconds: 2));
              await key.currentState!.setWidgetState(WidgetState.results);
            }
          },
        );
```
#### after calling display widget , if you need to change the state of the widget ,make sure to do the task with some delay in time,
#### we will be adding support for this future.

## To fetch List of Posted Widgets for a given Program
#### In order to fetch the posted widgets associated to a program , you need to call the getPostedWidget api of session class
```dart
final list = await session?.getPublishedWidgets(LiveLikePagination.first);
```
#### the LiveLikePagination enum is used for lazy loading, right now by default widgets loaded a limit of 20, in order 
#### to start the call the api with LiveLikePagination.first which will return you first page data and call api with LiveLikePagination.next which will get next page data and similarly LiveLikePagination.previous will get previous page data, in next once you get the empty array you have reached the last page.

## Fetch Widget Details
#### In order to fetch details of a particular widget 
```dart
await sdk.fetchWidgetDetails(widgetId!, widgetKind!);
```
#### The api requried widgetid and widgetKind , and the result will be LiveLikeWidget model class