import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:livelike_flutter_sdk/livelike_flutter_sdk.dart';

import 'home.dart';

class LeaderBoardScreen extends StatelessWidget {
  final controller = Get.put(LeaderBoardController());

  LeaderBoardScreen({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(),
      body: Container(
        padding: EdgeInsets.all(10),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Text(
              "LeaderBoards",
              style: TextStyle(fontWeight: FontWeight.bold),
            ),
            Expanded(
              child: Obx(
                () => controller.loadingLeaderBoard.value
                    ? Center(
                        child: CircularProgressIndicator(),
                      )
                    : ListView.separated(
                        itemBuilder: (context, index) {
                          final leaderBoard = controller.leaderBoards[index];
                          return ListTile(
                            title: Text("${leaderBoard.name}"),
                            selected: controller.selectedLeaderBoard.value ==
                                leaderBoard,
                            selectedTileColor: Colors.black12,
                            onTap: !controller.entryLoading.value
                                ? () {
                                    controller.selectedLeaderBoard.value =
                                        leaderBoard;
                                    controller.loadEntries();
                                  }
                                : null,
                          );
                        },
                        separatorBuilder: (context, index) => Divider(),
                        itemCount: controller.leaderBoards.length,
                      ),
              ),
            ),
            Obx(() => Row(
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                  children: [
                    Text(
                      "Entries",
                      style: TextStyle(fontWeight: FontWeight.bold),
                    ),
                    ElevatedButton(
                      onPressed: controller.selectedLeaderBoard.value != null &&
                              !controller.entryLoading.value
                          ? () {
                              controller.pagination = LiveLikePagination.first;
                              controller.loadEntries();
                            }
                          : null,
                      child: Text("FIRST"),
                    ),
                    ElevatedButton(
                      onPressed: controller.hasNext.value &&
                              !controller.entryLoading.value
                          ? () {
                              controller.pagination = LiveLikePagination.next;
                              controller.loadEntries();
                            }
                          : null,
                      child: Text("NEXT"),
                    ),
                    ElevatedButton(
                      onPressed: controller.hasPrevious.value &&
                              !controller.entryLoading.value
                          ? () {
                              controller.pagination =
                                  LiveLikePagination.previous;
                              controller.loadEntries();
                            }
                          : null,
                      child: Text("PREVIOUS"),
                    )
                  ],
                )),
            Expanded(
                child: Obx(() => controller.entryLoading.value
                    ? Center(
                        child: CircularProgressIndicator(),
                      )
                    : ListView.separated(
                        itemBuilder: (context, index) {
                          final entry = controller.leaderBoardEntries[index];
                          return ListTile(
                            title: Text("${entry.profile?.nickname}"),
                            subtitle: Text("Score: ${entry.score}"),
                            onTap: () {
                              controller.loadEntryDetail(entry: entry);
                            },
                          );
                        },
                        separatorBuilder: (context, index) => Divider(),
                        itemCount: controller.leaderBoardEntries.length)),
                flex: 2),
          ],
        ),
      ),
      floatingActionButton: Obx(
        () => controller.selectedLeaderBoard.value != null
            ? FloatingActionButton.extended(
                onPressed: !controller.entryLoading.value
                    ? () {
                        controller.loadEntryDetail();
                      }
                    : null,
                label: Text("Current User"),
              )
            : SizedBox.shrink(),
      ),
    );
  }
}

class LeaderBoardController extends GetxController {
  final leaderBoards = <LeaderBoard>[].obs;
  final sdk = Get.find<EngagementSDK>();
  var pagination = LiveLikePagination.first;
  final leaderBoardEntries = <LeaderBoardEntry>[].obs;
  final entryLoading = false.obs;
  final selectedLeaderBoard = Rx<LeaderBoard?>(null);
  final hasPrevious = false.obs;
  final hasNext = false.obs;
  final loadingLeaderBoard = false.obs;

  @override
  void onInit() async {
    super.onInit();
    final Channel channel = Get.arguments;
    loadingLeaderBoard.value = true;
    leaderBoards.addAll(await sdk.getLeaderBoards(channel.id!));
    loadingLeaderBoard.value = false;
  }

  void loadEntries() async {
    entryLoading.value = true;
    final result = await sdk.getEntriesForLeaderBoard(
        selectedLeaderBoard.value!.id, pagination);
    hasPrevious.value = result.hasPrevious;
    hasNext.value = result.hasNext;
    leaderBoardEntries
      ..clear()
      ..addAll(result.list);
    entryLoading.value = false;
  }

  void loadEntryDetail({LeaderBoardEntry? entry}) async {
    try {
      final result = entry != null
          ? await sdk.getLeaderBoardEntryForProfile(
              selectedLeaderBoard.value!.id, entry.profile_id!)
          : (selectedLeaderBoard.value != null
              ? await sdk.getLeaderBoardEntryForCurrentUserProfile(
                  selectedLeaderBoard.value!.id)
              : null);
      if (result != null)
        Get.dialog(Dialog(
          child: Container(
            padding: const EdgeInsets.all(10),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                Text("Name: ${result.profile?.nickname}"),
                Text("Score: ${result.score}"),
                Text("Rank: ${result.rank}"),
                Text("Percentile: ${result.percentile_rank}"),
              ],
            ),
          ),
        ));
    } catch (e) {
      Get.snackbar("LiveLike", "No User Record Found",
          backgroundColor: Colors.red, colorText: Colors.white);
    }
  }
}
