import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:livelike_flutter_sdk/livelike_flutter_sdk.dart';

class ProgramScreen extends StatelessWidget {
  final _controller = Get.put(ProgramController());

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(),
      body: Column(
        children: [
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: [
              ElevatedButton(
                  onPressed: () {
                    _controller.fetch(LiveLikePagination.first);
                  },
                  child: Text("First")),
              ElevatedButton(
                  onPressed: () {
                    _controller.fetch(LiveLikePagination.next);
                  },
                  child: Text("Next")),
              ElevatedButton(
                  onPressed: () {
                    _controller.fetch(LiveLikePagination.previous);
                  },
                  child: Text("Previous")),
            ],
          ),
          Expanded(
            child: Obx(
              () => _controller.loading.value
                  ? Center(
                      child: CircularProgressIndicator(),
                    )
                  : ListView.separated(
                      itemBuilder: (context, index) {
                        final program = _controller.programsList[index];
                        return ListTile(
                          title: Text("${program.title}"),
                          subtitle: Column(
                            children: [
                              Text("Client Id: ${program.clientId}"),
                              Text(
                                  "Application Url: ${program.applicationUrl}"),
                            ],
                          ),
                          onTap: () {
                            Get.defaultDialog(
                              title: "${program.title}",
                              content: SizedBox(
                                height: 560,
                                width: 320,
                                child: ListView(
                                  children: [
                                    Text("${program.toJson()}"),
                                  ],
                                ),
                              ),
                            );
                          },
                        );
                      },
                      separatorBuilder: (context, index) =>
                          Divider(thickness: 2, height: 15),
                      itemCount: _controller.programsList.length,
                    ),
            ),
          )
        ],
      ),
    );
  }
}

class ProgramController extends GetxController {
  final sdk = Get.find<EngagementSDK>();

  final programsList = <Program>[];
  final loading = false.obs;

  @override
  void onReady() {
    super.onReady();
    fetch(LiveLikePagination.first);
  }

  void fetch(LiveLikePagination pagination) async {
    loading.value = true;
    programsList
      ..clear()
      ..addAll(await sdk.fetchListOfProgram(pagination));
    loading.value = false;
  }
}
