import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:livelike_flutter_sdk/livelike_flutter_sdk.dart';
import 'package:livelike_flutter_sdk_example/home.dart';

class SponsorScreen extends StatelessWidget {
  SponsorScreen({Key? key}) : super(key: key);
  final _controller = Get.put(SponsorController());

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: Text("Sponsors")),
      body: Padding(
        padding: const EdgeInsets.all(15),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Expanded(
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  ElevatedButton(
                      onPressed: () {
                        _controller.fetchSponsors();
                      },
                      child: Text("Fetch Sponsor for ProgramId"))
                ],
              ),
            ),
            Expanded(
              flex: 3,
              child: Obx(() => _controller.sponsorLoading.value
                  ? Center(child: CircularProgressIndicator())
                  : ListView.separated(
                      itemBuilder: (context, index) {
                        final sponsor = _controller.sponsorsList[index];
                        return ListTile(
                          title: Text("${sponsor.name}"),
                          leading: Image.network(sponsor.logoUrl!,
                              height: 60, width: 60),
                        );
                      },
                      separatorBuilder: (context, index) => Divider(),
                      itemCount: _controller.sponsorsList.length)),
            )
          ],
        ),
      ),
    );
  }
}

class SponsorController extends GetxController {
  late Channel channel;
  final sdk = Get.find<EngagementSDK>();
  final sponsorsList = <SponsorModel>[];
  final sponsorLoading = false.obs;

  @override
  void onReady() {
    super.onReady();
    channel = Get.arguments;
  }

  void fetchSponsors() async {
    sponsorLoading.value = true;
    sponsorsList
      ..clear()
      ..addAll(await sdk.sponsor.fetchByProgramId(channel.id!));
    sponsorLoading.value = false;
  }
}
