import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:livelike_flutter_sdk/livelike_flutter_sdk.dart';

import 'home.dart';

class WidgetListScreen extends StatelessWidget {
  final _controller = Get.put(WidgetListController());

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        actions: [
          IconButton(
              onPressed: () {
                _controller.fetchData();
              },
              icon: Icon(Icons.help)),
        ],
      ),
      body: Obx(() => _controller.loading.value
          ? Center(child: CircularProgressIndicator())
          : ListView.separated(
              padding: const EdgeInsets.all(10),
              itemBuilder: (context, index) {
                final widgetView = _controller.widgetList.keys.elementAt(index);
                final widget = _controller.widgetList[widgetView];
                return widgetView;
              },
              itemCount: _controller.widgetList.keys.length,
              separatorBuilder: (context, index) => Divider(),
            )),
    );
  }
}

class WidgetListController extends GetxController {
  final sdk = Get.find<EngagementSDK>();

  // LiveLikeWidget? widget;
  final Map<WidgetView, LiveLikeWidget> widgetList =
      <WidgetView, LiveLikeWidget>{}.obs;
  Session? session;

  @override
  void onReady() async {
    super.onReady();
    final Channel channel = Get.arguments;
    session = await sdk.createSession(channel.id!);

    fetchData();
  }

  final loading = false.obs;

  void fetchData() async {
    loading.value = true;
    try {
      final list = await session?.getPublishedWidgets(LiveLikePagination.first);
      widgetList.clear();
      list?.forEach((widget) {
        // if (widget.kind == "emoji-slider") {
        final widgetKey = GlobalKey<WidgetViewState>();
        final widgetView = WidgetView(
          key: widgetKey,
          widgetListener: (id, kind) {
            // print("WidgetView>> $id >> $index");
          },
          showWidgetViewWithDefaultHeight: true,
          onWidgetViewInit: () {
            show(widgetKey, widget);
          },
        );
        widgetList[widgetView] = widget;
        // }
      });
    } catch (e) {
      print("error: $e");
    }
    loading.value = false;
  }

  void show(GlobalKey<WidgetViewState> key, LiveLikeWidget widget) async {
    if (key.currentState != null) {
      final List<Future<void>> list = [
        key.currentState!.enableDefaultWidgetTransition(false),
        key.currentState!.showDismissButton(false),
        key.currentState!.displayWidget(widget)
      ];
      await Future.wait(list);
      await Future.delayed(const Duration(seconds: 5));
      await key.currentState!.setWidgetState(WidgetState.results);
    }
  }
}
