import Flutter
import UIKit
import EngagementSDK


public class SwiftLivelikeFlutterSdkPlugin: NSObject, FlutterPlugin {
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "livelike_flutter_sdk", binaryMessenger: registrar.messenger())
        let instance = SwiftLivelikeFlutterSdkPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
        let factory = ChatViewFactory(messenger: registrar.messenger(),plugin:instance)
        registrar.register(factory, withId: "chatView")
        let widgetFactory = WidgetViewFactory(messenger: registrar.messenger(),plugin:instance)
        registrar.register(widgetFactory,withId:"widgetView")
        instance.accessTokenDelegate = FlutterEventChannel(name: "flutter_sdk_access_token_delegate",binaryMessenger:registrar.messenger())
        debugPrint("Add event Channel")
        instance.errorDelegate = FlutterEventChannel(name: "flutter_sdk_error_delegate", binaryMessenger: registrar.messenger())
        instance.rewardStreamChannel = FlutterEventChannel(name: "reward_stream", binaryMessenger: registrar.messenger())
    }
    
    var engagementSDK: EngagementSDK!
    var sessionList = [String: ContentSession]()
    var chatSessionList = [String: ChatSession]()
    var currentPDTTimeList = [String: Int]()
    var accessTokenDelegate:FlutterEventChannel!
    var errorDelegate:FlutterEventChannel!
    var rewardStreamChannel:FlutterEventChannel!
    private let errorHandler = EventStreamHandler()
    let rewardStreamHandler = EventStreamHandler()
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if (call.method == "getPlatformVersion") {
            result("iOS " + UIDevice.current.systemVersion)
        }else if(call.method=="initSDK"){
            if let args = call.arguments as? NSDictionary,
               let clientId = args["clientId"] as? String{
                let usingAccessToken: Bool = ((args["usingAccessToken"] as? Bool?) ?? false)!
                var config = EngagementSDKConfig(clientID:clientId)
                if ((usingAccessToken)){
                    if let accessToken = args["accessToken"] as? String{
                        config.accessTokenStorage = FlutterUserAccessTokenStorage(eventChannel: accessTokenDelegate,accessToken: accessToken)
                    }
                    else{
                        config.accessTokenStorage = FlutterUserAccessTokenStorage(eventChannel: accessTokenDelegate)
                    }
                }
                rewardStreamChannel.setStreamHandler(rewardStreamHandler)
                engagementSDK = EngagementSDK(config: config)
                errorDelegate.setStreamHandler(errorHandler)
                result(nil)
                return
            }
            result(FlutterError.init(code: "", message: "Client id cannot be empty", details: nil))
        }else if(call.method == "setVideoTimePDT"){
            if let args = call.arguments as? NSDictionary,
               let videoPDTTime = args["videoPDTTime"] as? Int{
                if let programId = args["programId"] as? String {
                    currentPDTTimeList[programId] = videoPDTTime
                }
                if let chatRoomId = args["chatRoomId"] as? String {
                    currentPDTTimeList[chatRoomId] = videoPDTTime
                }
            }
            result(nil)
        }else if(call.method=="createSession"){
            if let args = call.arguments as? NSDictionary,
               let programId = args["programId"] as? String{
                
                let config = SessionConfiguration(programID: programId)
                
                sessionList[programId] = engagementSDK.contentSession(config: config)
                
                sessionList[programId]!.getChatSession(completion: { [self] (chatResult) in
                    do {
                        let chatSession = try chatResult.get().self
                        let roomId = chatSession.roomID
                        chatSessionList[roomId] = chatSession
                        result(["programId":self.sessionList[programId]?.programID,"chatRoomId": roomId])
                    } catch {
                        errorHandler.eventSink?(error)
                        result(nil)
                    }
                })
                return
            }
            result (nil)
        }
        else if(call.method=="resumeSession"){
            if let args = call.arguments as? NSDictionary,
               let programId = args["programId"] as? String{
                sessionList[programId]?.resume()
                result(nil)
                return
            }
            result(FlutterError.init(code: "", message: "Error in resuming session", details: nil))
        }
        else if(call.method=="pauseSession"){
            if let args = call.arguments as? NSDictionary,
               let programId = args["programId"] as? String{
                sessionList[programId]?.pause()
                result(nil)
                return
            }
            result(FlutterError.init(code: "", message: "Error in pausing session", details: nil))
        }
        else if(call.method=="closeSession"){
            if let args = call.arguments as? NSDictionary,
               let programId = args["programId"] as? String{
                sessionList[programId]?.close()
                sessionList.removeValue(forKey: programId)
                result(nil)
                return
            }
            result(FlutterError.init(code: "", message: "Error in closing session", details: nil))
        }else if(call.method=="createChatSession"){
            if let args = call.arguments as? NSDictionary,
               let chatRoomId = args["chatRoomId"] as? String{
                let config = ChatSessionConfig(roomID: chatRoomId)
                self.engagementSDK.connectChatRoom(config: config) { [weak self] res in
                    guard let self = self else { return }
                    switch res {
                    case .success(let chatSession):
                        self.chatSessionList[chatRoomId] = chatSession
                        result(["chatRoomId":chatSession.roomID])
                    case .failure(let error):
                        self.errorHandler.eventSink?(error)
                    }
                }
                return
            }
            result(FlutterError.init(code: "", message: "Error in creating chat session", details: nil))
        }else if(call.method == "resumeChatSession"){
            if let args = call.arguments as? NSDictionary,
               let chatRoomId = args["chatRoomId"] as? String{
                chatSessionList[chatRoomId]?.resume()
                result(nil)
                return
            }
            result(FlutterError.init(code: "", message: "Error in resuming chat session", details: nil))
        }
        else if(call.method == "pauseChatSession"){
            if let args = call.arguments as? NSDictionary,
               let chatRoomId = args["chatRoomId"] as? String{
                chatSessionList[chatRoomId]?.pause()
                result(nil)
                return
            }
            result(FlutterError.init(code: "", message: "Error in oausing chat session", details: nil))
        }
        else if(call.method=="closeChatSession"){
            if let args = call.arguments as? NSDictionary,
               let chatRoomId = args["chatRoomId"] as? String{
                //                chatSessionList[chatRoomId]?.
                chatSessionList.removeValue(forKey: chatRoomId)
                result(nil)
                return
            }
            result(FlutterError.init(code: "", message: "Error in closing chat session", details: nil))
            
        } else if(call.method == "getLeaderBoards"){
            if let args = call.arguments as? NSDictionary,
               let programId = args["programId"] as? String{
                self.engagementSDK.getLeaderboards(programID: programId) {res in
                    switch(res){
                    case let .success(leaderboards):
                        result(leaderboards.map { (leaderboard) -> Dictionary<String, Any> in
                            return ["id":leaderboard.id,"name":leaderboard.name,"rewardItem":[
                                "id":leaderboard.rewardItem.id,
                                "name":leaderboard.rewardItem.name
                            ]]
                        })
                    case let .failure(error):
                        result(FlutterError.init(code: "",message: error.localizedDescription,details: nil))
                    }
                }
            }
            
        } else if(call.method == "getLeaderBoardDetails"){
            if let args = call.arguments as? NSDictionary,
               let leaderBoardId = args["leaderBoardId"] as? String{
                self.engagementSDK.getLeaderboard(leaderboardID: leaderBoardId) { res in
                    switch(res){
                    case let .success(leaderboard):
                        result(["id":leaderboard.id,"name":leaderboard.name,"rewardItem":[
                            "id":leaderboard.rewardItem.id,
                            "name":leaderboard.rewardItem.name
                        ]])
                    case let .failure(error):
                        result(FlutterError.init(code: "",message: error.localizedDescription,details: nil))
                    }
                }
            }
        }
        else if(call.method == "getLeaderBoardEntryForProfile"){
            if let args = call.arguments as? NSDictionary,
               let leaderBoardId = args["leaderBoardId"] as? String,
               let profileId = args["profileId"] as? String {
                self.engagementSDK.getLeaderboardEntry(profileID: profileId, leaderboardID: leaderBoardId) { res in
                    switch(res){
                    case let .success(leaderboard):
                        result(["profile_id":leaderboard.profileId,
                                "profile_nickname":leaderboard.profileNickname,
                                "percentile_rank": leaderboard.percentileRank,
                                "rank": leaderboard.rank,
                                "score":leaderboard.score,
                                "profile":[
                                    "id":leaderboard.profile.id,
                                    "nickname":leaderboard.profile.nickname,
                                    "custom_data":leaderboard.profile.customData
                                ]
                        ])
                    case let .failure(error):
                        result(FlutterError.init(code: "",message: error.localizedDescription,details: nil))
                    }
                }
                return
            }
            result(FlutterError.init(code: "",message: "LeaderBoard Id or Profile Id cannot be null",details: nil))
        }
        else if(call.method == "getLeaderBoardEntryForCurrentUserProfile"){
            if let args = call.arguments as? NSDictionary,
               let leaderBoardId = args["leaderBoardId"] as? String{
                self.engagementSDK.getLeaderboardEntryForCurrentProfile(leaderboardID: leaderBoardId) { res in
                    switch(res){
                    case let .success(leaderboard):
                        result(["profile_id":leaderboard.profileId,
                                "profile_nickname":leaderboard.profileNickname,
                                "percentile_rank": leaderboard.percentileRank,
                                "rank": leaderboard.rank,
                                "score":leaderboard.score,
                                "profile":[
                                    "id":leaderboard.profile.id,
                                    "nickname":leaderboard.profile.nickname,
                                    "custom_data":leaderboard.profile.customData
                                ]
                        ])
                    case let .failure(error):
                        result(FlutterError.init(code: "",message: error.localizedDescription,details: nil))
                    }
                }
                return
            }
            result(FlutterError.init(code: "",message: "LeaderBoard Id cannot be null",details: nil))
        }
        else if(call.method == "getEntriesForLeaderBoard"){
            if let args = call.arguments as? NSDictionary,
               let leaderBoardId = args["leaderBoardId"] as? String,
               let pagination = args["pagination"] as? String{
                var livelikePagination = Pagination.first
                if(pagination=="first"){
                    livelikePagination = Pagination.first
                }else if(pagination=="previous"){
                    livelikePagination = Pagination.previous
                }else if(pagination=="next"){
                    livelikePagination = Pagination.next
                }
                
                self.engagementSDK.getLeaderboardEntries(leaderboardID: leaderBoardId, page: livelikePagination){ res in
                    switch(res){
                    case let .success(data):
                        result([
                            "count": data.total,
                            "hasNext" : data.hasNext,
                            "hasPrevious" : data.hasPrevious,
                            "list": data.entries.map { (entry) -> Dictionary<String, Any> in
                                return ["percentile_rank":entry.percentileRank,
                                        "profile_id":entry.profileId,
                                        "profile":[
                                            "id":entry.profile.id,
                                            "nickname":entry.profile.nickname,
                                            "custom_data":entry.profile.customData
                                        ],
                                        "profile_nickname":entry.profileNickname,
                                        "score":entry.score,
                                        "rank":entry.rank
                                ]
                            }
                        ])
                    case let .failure(error):
                        result(FlutterError.init(code: "",message: error.localizedDescription,details: nil))
                    }
                }
                return
            }
            result(FlutterError.init(code: "",message: "LeaderBoard Id cannot be null",details: nil))
        }
        else if(call.method == "fetchSponsorByProgramId"){
            if let args = call.arguments as? NSDictionary,
               let programId = args["programId"] as? String{
                let sponsor = self.engagementSDK.sponsorship
                sponsor.getBy(programID: programId) { res in
                    switch(res){
                    case let .success(sponsors):
                        result(sponsors.map { (sponsor) -> Dictionary<String, Any> in
                            return ["id":sponsor.id,"name":sponsor.name,"logo_url": sponsor.logoURL.absoluteString,"client_id":sponsor.clientID]
                        })
                    case let .failure(error):
                        result(FlutterError.init(code: "",message: error.localizedDescription,details: nil))
                    }
                }
                return
            }
            result(FlutterError.init(code: "",message: "Program Id cannot be null",details: nil))
        } else if(call.method == "fetchWidgetDetails"){
            if let args = call.arguments as? NSDictionary,
               let widgetId = args["widgetId"] as? String,
               let widgetKind = args["widgetKind"] as? String{
                var widgetKindType: WidgetKind?
                switch widgetKind {
                case "text-prediction":
                    widgetKindType = WidgetKind.textPrediction
                    break
                case "text-prediction-follow-up":
                    widgetKindType = WidgetKind.textPredictionFollowUp
                    break
                case "image-prediction":
                    widgetKindType = WidgetKind.imagePrediction
                    break
                case "image-prediction-follow-up":
                    widgetKindType = WidgetKind.imagePredictionFollowUp
                    break
                case "image-poll":
                    widgetKindType = WidgetKind.imagePoll
                    break
                case "text-poll":
                    widgetKindType = WidgetKind.textPoll
                    break
                case "alert":
                    widgetKindType = WidgetKind.alert
                    break
                case "text-quiz":
                    widgetKindType = WidgetKind.textQuiz
                    break
                case "image-quiz":
                    widgetKindType = WidgetKind.imageQuiz
                    break
                case "emoji-slider":
                    widgetKindType = WidgetKind.imageSlider
                    break
                case "cheer-meter":
                    widgetKindType = WidgetKind.cheerMeter
                    break
                case "social-embed":
                    widgetKindType = WidgetKind.socialEmbed
                    break
                default:
                    break
                }
                let dateFormatter = DateFormatter()
                // initially set the format based on your datepicker date / server String
                engagementSDK.getWidget(id: widgetId, kind: widgetKindType!){ res in
                    switch(res){
                    case let .success(widget):
                        var data = [String:Any?]()
                        switch widget.widgetModel {
                        case let .alert(model):
                            data["program_id"] = model.programID;
                            data["created_at"] = dateFormatter.string(from:model.createdAt);
                            data["id"] = model.id;
                            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
                            data["kind"] = model.kind.rawValue;
                            data["custom_data"] = model.customData;
                            data["text"] = model.text;
                            data["image_url"] = model.imageURL?.absoluteString;
                            data["link_url"] = model.linkURL?.absoluteString;
                            data["link_label"] = model.linkLabel;
                            data["title"] = model.title;
                            data["timeout_time"] = Int(model.interactionTimeInterval)
                            break;
                        case let .cheerMeter(model):
                            data["program_id"] = model.programID;
                            data["created_at"] = dateFormatter.string(from:model.createdAt);
                            data["id"] = model.id;
                            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
                            data["kind"] = model.kind.rawValue;
                            data["custom_data"] = model.customData;
                            data["title"] = model.title
                            data["timeout_time"] = Int(model.interactionTimeInterval)
                            
                            var dict: [Dictionary<String, Any?>] = []
                            for option in model.options {
                                dict.append( ["id" : option.id,
                                              "image_url":option.imageURL.absoluteString,
                                              "text":option.text,
                                              "vote_count": option.voteCount
                                ])
                            }
                            data["options"] = dict
                            break;
                        case let .quiz(model):
                            data["program_id"] = model.programID;
                            data["created_at"] = dateFormatter.string(from:model.createdAt);
                            data["id"] = model.id;
                            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
                            data["kind"] = model.kind.rawValue;
                            data["custom_data"] = model.customData;
                            data["question"] = model.question;
                            data["timeout_time"] = Int(model.interactionTimeInterval)
                            
                            var dict: [Dictionary<String, Any?>] = []
                            for option in model.choices {
                                dict.append(["id" : option.id,
                                             "image_url":option.imageURL?.absoluteString,
                                             "text":option.text,
                                             "is_correct":option.isCorrect,
                                             "answer_count": option.answerCount
                                ])
                            }
                            data["choices"] = dict
                            break;
                        case let .prediction(model):
                            data["program_id"] = model.programID;
                            data["created_at"] = dateFormatter.string(from:model.createdAt);
                            data["id"] = model.id;
                            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
                            data["kind"] = model.kind.rawValue;
                            data["custom_data"] = model.customData;
                            data["question"] = model.question;
                            data["timeout_time"] = Int(model.interactionTimeInterval)
                            
                            var dict: [Dictionary<String, Any?>] = []
                            for option in model.options {
                                dict.append( ["id" : option.id,
                                              "image_url":option.imageURL?.absoluteString,
                                              "text":option.text,
                                              "vote_count": option.voteCount
                                ])
                            }
                            data["options"] = dict
                            break;
                        case let .predictionFollowUp(model):
                            data["program_id"] = model.programID;
                            data["created_at"] = dateFormatter.string(from:model.createdAt);
                            data["id"] = model.id;
                            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
                            data["kind"] = model.kind.rawValue;
                            data["custom_data"] = model.customData;
                            data["question"] = model.question;
                            data["timeout_time"] = Int(model.interactionTimeInterval)
                            
                            var dict: [Dictionary<String, Any?>] = []
                            for option in model.options {
                                dict.append(["id" : option.id,
                                             "image_url":option.imageURL?.absoluteString,
                                             "text":option.text,
                                             "vote_count": option.voteCount
                                ])
                            }
                            data["options"] = dict
                            break;
                        case  let .poll(model):
                            data["program_id"] = model.programID;
                            data["created_at"] = dateFormatter.string(from:model.createdAt);
                            data["id"] = model.id;
                            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
                            data["kind"] = model.kind.rawValue;
                            data["custom_data"] = model.customData;
                            data["question"] = model.question;
                            data["timeout_time"] = Int(model.interactionTimeInterval)
                            data["timeout"] = Int(model.interactionTimeInterval)
                            
                            var dict: [Dictionary<String, Any?>] = []
                            for option in model.options {
                                dict.append(["id" : option.id,
                                             "image_url":option.imageURL?.absoluteString,
                                             "text":option.text,
                                             "vote_count": option.voteCount
                                ])
                            }
                            data["options"] = dict
                            break;
                        case let .imageSlider(model):
                            data["program_id"] = model.programID;
                            data["created_at"] = dateFormatter.string(from:model.createdAt);
                            data["id"] = model.id;
                            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
                            data["kind"] = model.kind.rawValue;
                            data["custom_data"] = model.customData;
                            data["question"] = model.question;
                            var dict: [Dictionary<String, Any?>] = []
                            for option in model.options {
                                dict.append(["id" : option.id,
                                             "image_url":option.imageURL.absoluteString,
                                ])
                            }
                            data["options"] = dict
                            data["timeout_time"] = Int(model.interactionTimeInterval)
                            
                            data["initial_magnitude"] = model.initialMagnitude;
                            data["average_magnitude"] = model.averageMagnitude;
                            break;
                        case let .socialEmbed(model):
                            data["created_at"] = dateFormatter.string(from:model.createdAt);
                            data["id"] = model.id;
                            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
                            data["kind"] = model.kind.rawValue;
                            data["custom_data"] = model.customData;
                            data["comment"] = model.comment;
                            data["timeout_time"] = Int(model.interactionTimeInterval)
                            break;
                        case let .videoAlert(model):
                            data["program_id"] = model.programID;
                            data["created_at"] = dateFormatter.string(from:model.createdAt);
                            data["id"] = model.id;
                            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
                            data["kind"] = model.kind.rawValue;
                            data["custom_data"] = model.customData;
                            data["link_url"] = model.linkURL;
                            data["timeout_time"] = Int(model.interactionTimeInterval)
                            break;
                        case .none:
                            break;
                        case .some(.textAsk(_)):
                            break
                        }
                        result(data)
                        break
                    case let .failure(error):
                        result(FlutterError.init(code:"",message:error.localizedDescription,details:nil))
                        break
                    }
                    
                }
            }
        }
        else{
            result(FlutterMethodNotImplemented.`self`())
        }
    }
}

class FlutterUserAccessTokenStorage : AccessTokenStorage{
    private var accessToken:String?
    private let handler = EventStreamHandler()
    private var eventChannel : FlutterEventChannel?
    init(eventChannel:FlutterEventChannel,accessToken:String?=nil) {
        self.accessToken = accessToken
        self.eventChannel = eventChannel
        self.eventChannel!.setStreamHandler(handler)
    }
    
    func fetchAccessToken() -> String? {
        return self.accessToken
    }
    func storeAccessToken(accessToken: String) {
        self.handler.eventSink?(accessToken)
    }
}

class EventStreamHandler :NSObject, FlutterStreamHandler{
    private var _eventSink: FlutterEventSink?
    
    var eventSink: FlutterEventSink?{
        get {
            return _eventSink
        }
    }
    
    func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        _eventSink = events
        return nil
    }
    
    func onCancel(withArguments arguments: Any?) -> FlutterError? {
        _eventSink = nil
        return nil
    }
}

class ChatViewFactory: NSObject,FlutterPlatformViewFactory {
    private var messenger: FlutterBinaryMessenger
    private var plugin: SwiftLivelikeFlutterSdkPlugin
    init(messenger: FlutterBinaryMessenger,plugin:SwiftLivelikeFlutterSdkPlugin) {
        self.messenger = messenger
        self.plugin = plugin
        super.init()
    }
    
    func create(
        withFrame frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?
    ) -> FlutterPlatformView {
        return LLChatView(
            frame: frame,
            viewIdentifier: viewId,
            arguments: args,
            binaryMessenger: messenger,
            plugin:plugin
        )
    }
}

class LLChatView: NSObject, FlutterPlatformView {
    private var _view: UIView
    let chatViewController = ChatViewController()
    
    init(
        frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?,
        binaryMessenger : FlutterBinaryMessenger?,
        plugin:SwiftLivelikeFlutterSdkPlugin
    ) {
        _view = UIView()
        super.init()
        // iOS views can be created here
        createNativeView(view: _view)
        debugPrint("ChatViewID: chatView\(viewId)")
        let methodChannel = FlutterMethodChannel(name:"chatView\(viewId)",binaryMessenger: binaryMessenger!)
        methodChannel.setMethodCallHandler({
            (call: FlutterMethodCall, result: FlutterResult) -> Void in
            switch call.method{
            case "setSession":
                if let args = call.arguments as? Dictionary<String, Any>,
                   let chatRoomId = args["chatRoomId"] as? String{
                    if let session =  plugin.chatSessionList[chatRoomId]{
                        self.chatViewController.setChatSession(session)
                    } else {
                        result(FlutterError.init(code: "", message: "ChatSession not found", details: nil))
                    }
                }
                result(nil)
            default:
                result(FlutterMethodNotImplemented)
            }
        })
    }
    
    func view() -> UIView {
        return _view
    }
    
    func createNativeView(view _view: UIView){
        _view.addSubview(self.chatViewController.view)
        chatViewController.view.translatesAutoresizingMaskIntoConstraints = false
        NSLayoutConstraint.activate([
            chatViewController.view.bottomAnchor.constraint(equalTo: _view.bottomAnchor),
            chatViewController.view.topAnchor.constraint(equalTo: _view.topAnchor),
            chatViewController.view.trailingAnchor.constraint(equalTo: _view.trailingAnchor),
            chatViewController.view.leadingAnchor.constraint(equalTo: _view.leadingAnchor)
        ])
    }
}

class WidgetViewFactory: NSObject,FlutterPlatformViewFactory{
    private var messenger: FlutterBinaryMessenger
    private var plugin: SwiftLivelikeFlutterSdkPlugin
    init(messenger: FlutterBinaryMessenger,plugin:SwiftLivelikeFlutterSdkPlugin) {
        self.messenger = messenger
        self.plugin = plugin
        super.init()
    }
    
    func create(
        withFrame frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?
    ) -> FlutterPlatformView {
        return LLWidgetView(
            frame: frame,
            viewIdentifier: viewId,
            arguments: args,
            binaryMessenger: messenger,
            plugin:plugin
        )
    }
}

class LLWidgetView: NSObject, FlutterPlatformView, WidgetPopupViewControllerDelegate, LeaderboardDelegate {
    
    
    func widgetViewController(_ widgetViewController: WidgetPopupViewController, willDisplay widget: Widget) {
        print("will display => \(Int64(Date().timeIntervalSince1970 * 1000))> \(widget.kind.displayName) => \(_view.frame.height) => \(self.widgetViewController.currentWidget?.view.frame.height)")
        let data = getData(widgetModel: widget.widgetModel)
        widgetStreamHandler.eventSink?(["show": true,"data": data])
    }
    
    func widgetViewController(_ widgetViewController: WidgetPopupViewController, didDisplay widget: Widget) {
        print("did display => \(Int64(Date().timeIntervalSince1970 * 1000))> \(widget.kind.displayName) => \(self.widgetViewController.currentWidget?.view.frame.height)")
        let data = getData(widgetModel: widget.widgetModel)
        widgetStreamHandler.eventSink?(["show": true,"widget_height":self.widgetViewController.currentWidget?.view?.frame.height ?? 0,"data":data])
    }
    
    func widgetViewController(_ widgetViewController: WidgetPopupViewController, willDismiss widget: Widget) {
        print("will dismiss => \(Int64(Date().timeIntervalSince1970 * 1000))> \(widget.kind.displayName)")
    }
    
    func widgetViewController(_ widgetViewController: WidgetPopupViewController, didDismiss widget: Widget) {
        print("did dismiss => \(Int64(Date().timeIntervalSince1970 * 1000))> \(widget.kind.displayName)")
        let data = getData(widgetModel: widget.widgetModel)
        widgetStreamHandler.eventSink?(["show":false,"data":data])
    }
    
    func widgetViewController(_ widgetViewController: WidgetPopupViewController, willEnqueueWidget widgetModel: WidgetModel) -> Widget? {
        print("will enqueu => \(widgetModel) >> \(Int64(Date().timeIntervalSince1970 * 1000))")
        let widget = DefaultWidgetFactory.makeWidget(from: widgetModel)
        return widget
    }
    
    
    func getData(widgetModel:WidgetModel?) -> [String:Any?] {
        let dateFormatter = DateFormatter()
        var data = [String:Any?]()
        switch widgetModel {
        case let .alert(model):
            data["program_id"] = model.programID;
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["text"] = model.text;
            data["image_url"] = model.imageURL?.absoluteString;
            data["link_url"] = model.linkURL?.absoluteString;
            data["link_label"] = model.linkLabel;
            data["title"] = model.title;
            data["timeout_time"] = Int(model.interactionTimeInterval)
            break;
        case let .cheerMeter(model):
            data["program_id"] = model.programID;
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["title"] = model.title
            data["timeout_time"] = Int(model.interactionTimeInterval)
            
            var dict: [Dictionary<String, Any?>] = []
            for option in model.options {
                dict.append( ["id" : option.id,
                              "image_url":option.imageURL.absoluteString,
                              "text":option.text,
                              "vote_count": option.voteCount
                ])
            }
            data["options"] = dict
            break;
        case let .quiz(model):
            data["program_id"] = model.programID;
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["question"] = model.question;
            data["timeout_time"] = Int(model.interactionTimeInterval)
            
            var dict: [Dictionary<String, Any?>] = []
            for option in model.choices {
                dict.append(["id" : option.id,
                             "image_url":option.imageURL?.absoluteString,
                             "text":option.text,
                             "is_correct":option.isCorrect,
                             "answer_count": option.answerCount
                ])
            }
            data["choices"] = dict
            break;
        case let .prediction(model):
            data["program_id"] = model.programID;
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["question"] = model.question;
            data["timeout_time"] = Int(model.interactionTimeInterval)
            
            var dict: [Dictionary<String, Any?>] = []
            for option in model.options {
                dict.append( ["id" : option.id,
                              "image_url":option.imageURL?.absoluteString,
                              "text":option.text,
                              "vote_count": option.voteCount
                ])
            }
            data["options"] = dict
            break;
        case let .predictionFollowUp(model):
            data["program_id"] = model.programID;
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["question"] = model.question;
            data["timeout_time"] = Int(model.interactionTimeInterval)
            
            var dict: [Dictionary<String, Any?>] = []
            for option in model.options {
                dict.append(["id" : option.id,
                             "image_url":option.imageURL?.absoluteString,
                             "text":option.text,
                             "vote_count": option.voteCount
                ])
            }
            data["options"] = dict
            break;
        case  let .poll(model):
            data["program_id"] = model.programID;
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["question"] = model.question;
            data["timeout_time"] = Int(model.interactionTimeInterval)
            
            var dict: [Dictionary<String, Any?>] = []
            for option in model.options {
                dict.append(["id" : option.id,
                             "image_url":option.imageURL?.absoluteString,
                             "text":option.text,
                             "vote_count": option.voteCount
                ])
            }
            data["options"] = dict
            break;
        case let .imageSlider(model):
            data["program_id"] = model.programID;
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["question"] = model.question;
            var dict: [Dictionary<String, Any?>] = []
            for option in model.options {
                dict.append(["id" : option.id,
                             "image_url":option.imageURL.absoluteString,
                ])
            }
            data["options"] = dict
            data["timeout_time"] = Int(model.interactionTimeInterval)
            
            data["initial_magnitude"] = model.initialMagnitude;
            data["average_magnitude"] = model.averageMagnitude;
            break;
        case let .socialEmbed(model):
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["comment"] = model.comment;
            data["timeout_time"] = Int(model.interactionTimeInterval)
            break;
        case let .videoAlert(model):
            data["program_id"] = model.programID;
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["link_url"] = model.linkURL;
            data["timeout_time"] = Int(model.interactionTimeInterval)
            break;
        case .textAsk(_):
            break
        case .none:
            break;
        }
        return data
    }
    
    private var _view: UIView
    let widgetViewController = WidgetPopupViewController()
    var rewardStreamHandler: EventStreamHandler?
    var widgetStreamChannel:FlutterEventChannel?
    let widgetStreamHandler = EventStreamHandler()
    
    func leaderboard(_ leaderboardClient: LeaderboardClient, currentPositionDidChange position: LeaderboardPosition) {
        //        debugPrint("Rewards: \(position.percentileRank), \(position.rank) , \(position.score) ")
    }
    
    var widgetArray = [Widget]()
    
    init(
        frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?,
        binaryMessenger : FlutterBinaryMessenger?,
        plugin:SwiftLivelikeFlutterSdkPlugin
    ) {
        _view = UIView()
        super.init()
        widgetStreamChannel = FlutterEventChannel(name: "widget_stream_\(viewId)", binaryMessenger: binaryMessenger!)
        widgetStreamChannel!.setStreamHandler(widgetStreamHandler)
        // iOS views can be created here
       
        debugPrint("WidgetView ID : widgetView\(viewId)")
        self.rewardStreamHandler = plugin.rewardStreamHandler
        let methodChannel = FlutterMethodChannel(name:"widgetView\(viewId)",binaryMessenger: binaryMessenger!)
        methodChannel.setMethodCallHandler({(call:FlutterMethodCall, result:FlutterResult) -> Void in
        })
        
        methodChannel.setMethodCallHandler({
            (call: FlutterMethodCall, result: FlutterResult) -> Void in
            switch call.method {
            case "setSession":
                if let args = call.arguments as? Dictionary<String, Any>,
                   let programId = args["programId"] as? String {
                    self.createNativeView()
                    let session =  plugin.sessionList[programId]
                    self.widgetViewController.session  = session
                    self.widgetViewController.delegate = self
                }
                result(nil)
                break
            case "setState":
                if let args = call.arguments as? Dictionary<String, Any>,
                   let state = args["state"] as? String {
                    switch state {
                    case "ready":
                        self.widgetArray.first?.currentState = .ready
                        break
                    case "interacting":
                        self.widgetArray.first?.currentState = .interacting
                        break
                    case "results":
                        self.widgetArray.first?.currentState = .results
                        break
                    case "finished":
                        self.widgetArray.first?.currentState = .finished
                        break
                    default:
                        self.widgetArray.first?.currentState = .ready
                    }
                    print("CHECK state: \(self.widgetArray.first?.currentState)")
                }
                break
            case "enableDefaultWidgetTransition" :
                if let args = call.arguments as? Dictionary<String,Any>,
                   let enable = args["enable"] as? Bool {
                    //                    self.widgetViewController.
                    result(nil)
                }
                break
            case "displayWidget":
                if var jsonText = call.arguments as? String {
                    var dictonary: [String:Any]?
                   
                    if let data = jsonText.data(using: String.Encoding.utf8) {
                        
                        do {
                            dictonary = try JSONSerialization.jsonObject(with: data, options: []) as? [String:Any]
                            
                            if let myDictionary = dictonary
                            {
                                print("CHECK ID is: \(myDictionary["id"]!)")
                                plugin.engagementSDK.createWidget(withJSONObject: myDictionary) {[weak self] res in
                                    guard let self = self else { return }
                                    switch (res) {
                                        case let .success(widget):
                                            
                                            self.widgetArray.append(widget)
                                            
                                            switch widget.widgetModel {
                                            case let .cheerMeter(model):
                                                model.loadInteractionHistory { res in
                                                    self.addwidgetToView(widget: widget)
                                                }
                                                break;
                                            case let .quiz(model):
                                                model.loadInteractionHistory {  res in
                                                    self.addwidgetToView(widget: widget)
                                                }
                                                break;
                                            case let .prediction(model):
                                                model.loadInteractionHistory { res in
                                                    self.addwidgetToView(widget: widget)
                                                }
                                                break;
                                            case let .predictionFollowUp(model):
                                                model.loadInteractionHistory { res in
                                                    self.addwidgetToView(widget: widget)
                                                }
                                                break;
                                            case  let .poll(model):
                                                model.loadInteractionHistory { res in
                                                    self.addwidgetToView(widget: widget)
                                                }
                                                break;
                                            case let .imageSlider(model):
                                                model.loadInteractionHistory { res in
                                                    self.addwidgetToView(widget: widget)
                                                }
                                                break;
                                            case let .textAsk(model):
                                                model.loadInteractionHistory { res in
                                                    self.addwidgetToView(widget: widget)
                                                }
                                                break
                                            case .none:
                                                break;
                                            default:
                                                self.addwidgetToView(widget: widget)
                                                break;
                                            }
                                            
                                            break
                                        case let .failure(error):
                                            print("CHECK error \(error.localizedDescription) => \(dictonary?["kind"] as? String)")
                                            break
                                        }
                                    
                                }
                            }
//                           result(nil)
                        } catch let error as NSError {
                            print(error)
                            result(nil)
                        }
                    }else{
                        result(nil)
                    }
                }else{
                    result(nil)
                }
                //json string
                break
            default:
                result(FlutterMethodNotImplemented)
            }
        })
    }
    
    func addwidgetToView(widget:Widget) -> Void {
        self._view.addSubview(widget.view)
        self._view.isUserInteractionEnabled = true
        print("CHECK Widget > \(widget.kind)")
        
        widget.view.translatesAutoresizingMaskIntoConstraints = false
        NSLayoutConstraint.activate([
//            widget.view.bottomAnchor.constraint(equalTo: self._view.bottomAnchor),
            widget.view.topAnchor.constraint(equalTo: self._view.topAnchor),
            widget.view.trailingAnchor.constraint(equalTo: self._view.trailingAnchor),
            widget.view.leadingAnchor.constraint(equalTo: self._view.leadingAnchor),
//            widget.view.heightAnchor.constraint(greaterThanOrEqualToConstant: 0)
        ])
//        DispatchQueue.main.async {
            let data = self.getData(widgetModel: widget.widgetModel)
        widget.view?.layoutIfNeeded()
        print("CHECK current state1 \(widget.currentState)")
        print("CHECK current state \(widget.currentState) => \(widget.view?.frame.height ?? 0) => \(data["kind"] as? String) => \(widgetArray.first?.view?.frame.height ?? 0)")
        self.widgetStreamHandler.eventSink?(["show": true,"widget_height":(widget.view?.frame.height ?? 0),"data":data])

//        }
    }
    
    func view() -> UIView {
        return _view
    }
    
    func createNativeView() {
        self._view.addSubview(self.widgetViewController.view)
        // Applying constraints to fill the container
        widgetViewController.view.translatesAutoresizingMaskIntoConstraints = false
        NSLayoutConstraint.activate([
            widgetViewController.view.bottomAnchor.constraint(equalTo:  self._view.bottomAnchor),
            widgetViewController.view.topAnchor.constraint(equalTo:  self._view.topAnchor),
            widgetViewController.view.trailingAnchor.constraint(equalTo:  self._view.trailingAnchor),
            widgetViewController.view.leadingAnchor.constraint(equalTo:  self._view.leadingAnchor),
        ])
        self.widgetViewController.view?.layoutIfNeeded()
    }
}
