import 'dart:convert';

import 'package:http/http.dart' as http;
import 'package:livelike_flutter_sdk/livelike_flutter_sdk.dart';
import 'package:livelike_flutter_sdk/src/model/timeline_response.dart';

import '../constants.dart';

class WidgetRepository {
  Future<LiveLikeWidget> getWidgetDetails(
      String widgetId, String widgetKind) async {
    final response =
        await http.get(Uri.parse(baseUrl + "widgets/$widgetKind/$widgetId"));
    return LiveLikeWidget.fromJson(jsonDecode(response.body));
  }

  Future<WidgetUserInteraction?> getWidgetInteractionDetails(String widgetId,
      String widgetKind, String profileId, String accessToken) async {
    final widget = await getWidgetDetails(widgetId, widgetKind);
    final response = await http.get(
      Uri.parse(widget.widget_interactions_url_template!
          .replaceFirst("{profile_id}", profileId)),
      headers: {"Authorization": "Bearer $accessToken"},
    );
    Map data = jsonDecode(response.body);
    Map interations = data["interactions"];

    List widgetInteractions = interations[widgetKind.contains("follow-up")
            ? widgetKind.replaceFirst("-follow-up", "")
            : widgetKind] ??
        [];
    if (widgetInteractions.isEmpty) {
      return null;
    }
    if (widgetKind == "cheer-meter") {
      return widgetInteractions
          .map((e) => WidgetUserInteraction.fromJson(e))
          .reduce((value, element) {
        value.vote_count = (value.vote_count ?? 0) + (element.vote_count ?? 0);
        return value;
      });
    }
    return WidgetUserInteraction.fromJson(widgetInteractions.first);
  }

  Future<TimeLineResponse> getPublishedWidgets(
      String programDetailTemplateUrl, String programId, String accessToken,
      {String? url}) async {
    String? timeLineUrl;
    if (url == null) {
      final programResponse = await http.get(
        Uri.parse(
            programDetailTemplateUrl.replaceFirst("{program_id}", programId)),
        headers: {"Authorization": "Bearer $accessToken"},
      );
      Map<String, dynamic> programData = jsonDecode(programResponse.body);
      timeLineUrl = programData["timeline_url"];
    }
    final response = await http.get(Uri.parse(url ?? timeLineUrl!));
    return TimeLineResponse.fromJson(jsonDecode(response.body));
  }
}
