import 'package:livelike_flutter_sdk/livelike_flutter_sdk.dart';
import 'package:livelike_flutter_sdk/src/constants.dart';
import 'package:livelike_flutter_sdk/src/model/livelike_widget.dart';
import 'package:livelike_flutter_sdk/src/model/timeline_response.dart';
import 'package:livelike_flutter_sdk/src/repository/widget_repository.dart';

class Session extends BaseSession {
  final String? programId;
  ChatSession? chatSession;
  final _widgetRepository = WidgetRepository();
  Stream _accessTokenStream = accessTokenEventChannel.receiveBroadcastStream();
  final String? _accessToken;
  final String _programDetailTemplateUrl;

  Session(this.programId, String? chatRoomId, this._accessToken,
      this._programDetailTemplateUrl)
      : super(programId) {
    chatSession = ChatSession(chatRoomId);
  }

  @override
  Future<void> pause() {
    return livelike_method_channel
        .invokeMethod(pauseSessionKey, {programIDKey: programId});
  }

  @override
  Future<void> resume() {
    return livelike_method_channel
        .invokeMethod(resumeSessionKey, {programIDKey: programId});
  }

  @override
  Future<void> close() {
    return livelike_method_channel
        .invokeMethod(closeSessionKey, {programIDKey: programId});
  }

  TimeLineResponse? _timeLineResponse;

  Future<List<LiveLikeWidget>> getPublishedWidgets(
      LiveLikePagination pagination) async {
    String? url;
    switch (pagination) {
      case LiveLikePagination.first:
        url = null;
        break;
      case LiveLikePagination.next:
        url = _timeLineResponse?.next;
        break;
      case LiveLikePagination.previous:
        url = _timeLineResponse?.previous;
        break;
      default:
        url = null;
    }
    _timeLineResponse = await _widgetRepository.getPublishedWidgets(
        _programDetailTemplateUrl,
        programId!,
        _accessToken ?? await _accessTokenStream.first,
        url: url);
    return _timeLineResponse?.results ?? <LiveLikeWidget>[];
  }
}

class ChatSession extends BaseSession {
  final String? chatRoomId;

  ChatSession(this.chatRoomId) : super(chatRoomId);

  @override
  Future<void> close() {
    return livelike_method_channel
        .invokeMethod(closeSessionKey, {chatRoomIdKey: chatRoomId});
  }

  @override
  Future<void> pause() {
    return livelike_method_channel
        .invokeMethod(pauseChatSessionKey, {chatRoomIdKey: chatRoomId});
  }

  @override
  Future<void> resume() {
    return livelike_method_channel
        .invokeMethod(resumeChatSessionKey, {chatRoomIdKey: chatRoomId});
  }
}

abstract class BaseSession {
  String? id;

  BaseSession(this.id);

  Future<void> setVideoPDTTime(int time) {
    return livelike_method_channel.invokeMethod(setVideoTimePDTKey, {
      ((this is ChatSession) ? chatRoomIdKey : programIDKey): id,
      videoTimePDTKey: time,
    });
  }

  Future<void> resume();

  Future<void> pause();

  Future<void> close();
}
